unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, nh323types, nh323core, nh323conn, StdCtrls, ExtCtrls, Buttons;

type
  TfrmDemo = class(TForm)
    h323: TNH323Core;
    pnlInfo: TPanel;
    Label5: TLabel;
    eNATRouter: TEdit;
    Label2: TLabel;
    eLocalUserName: TEdit;
    cUseGatekeeper: TCheckBox;
    Label4: TLabel;
    eGatekeeperHost: TEdit;
    eGatekeeperPassword: TEdit;
    Label3: TLabel;
    cNATTranslation: TCheckBox;
    cFastStart: TCheckBox;
    cH245Tunneling: TCheckBox;
    Label6: TLabel;
    cBandwidth: TComboBox;
    Label7: TLabel;
    eListenerAddress: TEdit;
    Label8: TLabel;
    cUserInputMode: TComboBox;
    Label9: TLabel;
    cSilenceDetect: TComboBox;
    Label10: TLabel;
    Label11: TLabel;
    eRTPMin: TEdit;
    eRTPMax: TEdit;
    Label12: TLabel;
    eUDPBase: TEdit;
    Label13: TLabel;
    eUDPMax: TEdit;
    Label14: TLabel;
    eTCPBase: TEdit;
    Label15: TLabel;
    eTCPMax: TEdit;
    Label20: TLabel;
    eRTPTos: TEdit;
    eTraceFile: TEdit;
    pnlAudio: TPanel;
    Label16: TLabel;
    ePlay: TEdit;
    Label17: TLabel;
    eJitMin: TEdit;
    eJitMax: TEdit;
    Label18: TLabel;
    eRecord: TEdit;
    Label19: TLabel;
    Label21: TLabel;
    eBufferDepth: TEdit;
    Label22: TLabel;
    cCodecPref: TComboBox;
    Label24: TLabel;
    Label25: TLabel;
    cTraceLevel: TComboBox;
    pnlCmd: TPanel;
    Label1: TLabel;
    btnAnswer: TButton;
    Log: TMemo;
    eDialNumber: TEdit;
    btnDial: TButton;
    btnClear: TButton;
    Label23: TLabel;
    btnStart: TBitBtn;
    btnStop: TBitBtn;
    Bevel1: TBevel;
    btnAudioUpdate: TSpeedButton;
    Bevel2: TBevel;
    procedure FormCreate(Sender: TObject);
    procedure cUseGatekeeperClick(Sender: TObject);
    procedure btnStartClick(Sender: TObject);
    procedure btnStopClick(Sender: TObject);
    procedure h323GatekeeperConnect(AStatus: Cardinal; AGKAddress,
      AGKUsername, AGKPassword: String);
    procedure h323GatekeeperRegistration(AStatus: Cardinal; AGKAddress,
      AGKUsername, AGKPassword: String);
    procedure h323Alerting(AToken: String; var AlertStatus: LongBool);
    procedure h323AnswerCall(AToken, ACaller: String;
      var AnswerCallResponse: Th323AnswerCallResponses);
    procedure h323ConnectionCleared(AToken: String);
    procedure h323ConnectionEstablished(AToken: String);
    procedure h323IncomingCall(AToken: String; var Accept: LongBool);
    procedure h323OutgoingCall(AToken: String; var Accept: LongBool);
    procedure btnAnswerClick(Sender: TObject);
    procedure btnClearClick(Sender: TObject);
    procedure btnDialClick(Sender: TObject);
    procedure btnAudioUpdateClick(Sender: TObject);
  private
    { Private declarations }
    fAnswerNeeded : Boolean;
    fCallInProgress : Boolean;
    fToken : String;

    procedure SetDefaults;
    procedure GetAvailCodecs;
    procedure UpdateControls;

    procedure Set323Config;

  public
    { Public declarations }
  end;

var
  frmDemo: TfrmDemo;

implementation

{$R *.dfm}

procedure TfrmDemo.SetDefaults;
begin
  fCallInProgress := False;

  eListenerAddress.Text := '*';

  cUseGatekeeper.Checked := False;
  eGatekeeperHost.Text := '';
  eGatekeeperPassword.Text := '';
  cBandwidth.ItemIndex := 3;

  cNATTranslation.Checked := False;
  eNATRouter.Text := '';

  cFastStart.Checked := False;
  cH245Tunneling.Checked := False;

  cUserInputMode.ItemIndex:= 0;
  cSilenceDetect.ItemIndex := 0;

  eRTPTos.Text := '176';
  eRTPMin.Text := '5000';
  eRTPMax.Text := '5004';
  eUDPBase.Text := '5005';
  eUDPMax.Text := '5010';
  eTCPBase.Text := '5005';
  eTCPMax.Text := '5010';

  cTraceLevel.ItemIndex := 0;
  eTraceFile.Text := ExtractFilePath(Application.ExeName)+'nh323core.log';

  ePlay.Text := '<Your Audio Device Name>';
  eRecord.Text := ePlay.Text;

  eJitMin.Text := '50';
  eJitMax.Text := '250';

  GetAvailCodecs;

end;

procedure TfrmDemo.GetAvailCodecs;
var
  i : integer;
begin
  cCodecPref.Items.Clear;
  cCodecPref.Items.Add('Default');
  for i := 0 to NH323_CODECS_COUNT - 1 do
    cCodecPref.Items.Add( NH323_CODECS[i] );
  cCodecPref.ItemIndex := 0;
end;


procedure TfrmDemo.UpdateControls;
begin
  eGatekeeperHost.Enabled := cUseGatekeeper.Checked;
  eGatekeeperPassword.Enabled := cUseGatekeeper.Checked;
  cBandwidth.Enabled := cUseGatekeeper.Checked;

  eNATRouter.Enabled := cNATTranslation.Checked;

  eTraceFile.Enabled := (cTraceLevel.ItemIndex <> 0);


  eDialNumber.Enabled := h323.Active;
  btnClear.Enabled := h323.Active;
  btnAnswer.Enabled := h323.Active;

  btnStart.Enabled := not h323.Active;
  btnStop.Enabled := h323.Active;

  pnlInfo.Enabled := not h323.Active;
  cCodecPref.Enabled := not h323.Active;

  btnAudioUpdate.Enabled := h323.Active;

  btnAnswer.Enabled := fAnswerNeeded;
  btnClear.Enabled := fCallInProgress;
  btnDial.Enabled := not fCallInProgress and h323.Active;
end;



procedure TfrmDemo.FormCreate(Sender: TObject);
begin
  SetDefaults;
  UpdateControls;
end;

procedure TfrmDemo.cUseGatekeeperClick(Sender: TObject);
begin
  UpdateControls;
end;


procedure TfrmDemo.Set323Config;
begin
  h323.LocalUserName := eLocalUserName.Text;
  h323.ListenerAddress := eListenerAddress.Text;

  if cUseGatekeeper.Checked then
  begin
    h323.UseGatekeeper := hmUseGatekeeper;
    h323.GatekeeperAddress := eGatekeeperHost.Text;
    h323.GatekeeperPassword := eGatekeeperPassword.Text;
    case cBandwidth.ItemIndex of
      0: h323.InitialBandwidth := hbModem14k;
      1: h323.InitialBandwidth :=hbModem28k;
      2: h323.InitialBandwidth :=hbModem56k;
      3: h323.InitialBandwidth :=hb64kISDN;
      4: h323.InitialBandwidth :=hb128kISDN;
      5: h323.InitialBandwidth :=hbDSL;
      6: h323.InitialBandwidth :=hbLAN;
    end;  
  end else
    h323.UseGatekeeper := hmNoGatekeeper;


  if cNATTranslation.Checked then
  begin
    h323.NatTranslation := hmEnabled;
    h323.NatRouterAddress := eNATRouter.Text;
  end else
    h323.NatTranslation := hmDisabled;

  if cFastStart.Checked then
    h323.FastStart := hmEnabled
  else
    h323.FastStart := hmDisabled;

  if cH245Tunneling.Checked then
    h323.H245Tunneling  := hmEnabled
  else
    h323.H245Tunneling := hmDisabled;

  case cUserInputMode.ItemIndex of
    0: h323.SendUserInputMode := huimAsQ931;
    1: h323.SendUserInputMode := huimAsString;
    2: h323.SendUserInputMode := huimAsTone;
    3: h323.SendUserInputMode := huimAsInlineRFC2833;
  end;

  case cSilenceDetect.ItemIndex of
    0: h323.SilenceDetectionMode := hsmNoSilenceDetection;
    1: h323.SilenceDetectionMode := hsmFixedSilenceDetection;
    2: h323.SilenceDetectionMode := hsmAdaptiveSilenceDetection;
  end;

  h323.RTPTos := StrToInt( eRTPTos.Text );
  h323.RTPPortMin := StrToInt( eRTPMin.Text );
  h323.RTPPortMax := StrToInt( eRTPMax.Text );

  h323.UDPPortMin := StrToInt( eUDPBase.Text );
  h323.UDPPortMax := StrToInt( eUDPMax.Text );

  h323.TCPPortMin := StrToInt( eTCPBase.Text );
  h323.TCPPortMax := StrToInt( eTCPMax.Text );

  case cTraceLevel.ItemIndex of
    0: h323.TraceLevel := tNoTrace;
    1: h323.TraceLevel := tL1;
    2: h323.TraceLevel := tL2;
    3: h323.TraceLevel := tL3;
    4: h323.TraceLevel := tL4;
    5: h323.TraceLevel := tL5;
    6: h323.TraceLevel := tL6;
    7: h323.TraceLevel := tL7;
    8: h323.TraceLevel := tL8;
    9: h323.TraceLevel := tL9;
  end;
  if cTraceLevel.ItemIndex > 0 then
    h323.TraceFile := eTraceFile.Text;

  h323.SoundChannelPlayDevice := ePlay.Text;  
  h323.SoundChannelRecordDevice := eRecord.Text;

  h323.AudioJitterDelayMin := StrToInt( eJitMin.Text );  
  h323.AudioJitterDelayMax := StrToInt( eJitMax.Text );
  h323.SoundChannelBufferDepth := StrToInt( eBufferDepth.Text );

  if cCodecPref.ItemIndex > 0 then
  begin
    h323.AudioCodecs.Clear;
    h323.AudioCodecs.Add(cCodecPref.Items[cCodecPref.ItemIndex]);
  end;   

end;

procedure TfrmDemo.btnStartClick(Sender: TObject);
begin
  Set323Config;
  h323.Active := True;
  UpdateControls;
end;



procedure TfrmDemo.btnStopClick(Sender: TObject);
begin
  h323.Active := False;
  UpdateControls;
//  Close;
end;
//***********************
//***********************
//***********************
//***********************





procedure TfrmDemo.h323Alerting(AToken: String; var AlertStatus: LongBool);
begin
  fToken := AToken;
  Log.Lines.Add('OnAlering: Token='+AToken);
  AlertStatus := True;
end;

procedure TfrmDemo.h323AnswerCall(AToken, ACaller: String;
  var AnswerCallResponse: Th323AnswerCallResponses);
begin
  fToken := AToken;
  fAnswerNeeded := True;
  Log.Lines.Add('OnAnswerCall: Token='+AToken+', Caller='+ACaller);
  UpdateControls;
  AnswerCallResponse := acrAlertWithMedia;
end;

procedure TfrmDemo.h323ConnectionCleared(AToken: String);
begin
  fToken := AToken;
  fCallInProgress := False;
  fAnswerNeeded := False;
  UpdateControls;
  Log.Lines.Add('OnConnectionCleared: Token='+AToken);
end;

procedure TfrmDemo.h323ConnectionEstablished(AToken: String);
begin
  fCallInProgress := True;
  fAnswerNeeded := False;
  UpdateControls;
  Log.Lines.Add('OnConnectionEstablished: Token='+AToken);
end;

procedure TfrmDemo.h323GatekeeperConnect(AStatus: Cardinal; AGKAddress, AGKUsername, AGKPassword: String);
var
  sMsg : String;
begin
  sMsg := 'OnGatekeeperConnect: Status=';
  if AStatus = HSTATUS_CONFIRM then
    sMsg := sMsg + 'CONFIRMED, '
  else
    sMsg := sMsg + 'REJECTED, ';
  sMsg := sMsg + 'Host='+AGKAddress;
  Log.Lines.Add(sMsg);
end;

procedure TfrmDemo.h323GatekeeperRegistration(AStatus: Cardinal;
  AGKAddress, AGKUsername, AGKPassword: String);
var
  sMsg : String;
begin
  sMsg := 'OnGatekeeperRegistration: Status=';
  if AStatus = HSTATUS_CONFIRM then
    sMsg := sMsg + 'CONFIRMED, '
  else
    sMsg := sMsg + 'REJECTED, ';
  sMsg := sMsg + 'Host='+AGKAddress;
  Log.Lines.Add(sMsg);
end;



procedure TfrmDemo.h323IncomingCall(AToken: String; var Accept: LongBool);
begin
  Log.Lines.Add('OnIncomingCall: Token='+AToken);
  Accept := True;
end;

procedure TfrmDemo.h323OutgoingCall(AToken: String; var Accept: LongBool);
begin
  Log.Lines.Add('OnOutgoingCall: Token='+AToken);
  Accept := True;
end;

procedure TfrmDemo.btnAnswerClick(Sender: TObject);
begin
  if fAnswerNeeded then
    h323.AnsweringCall(fToken, acrNow);
end;

procedure TfrmDemo.btnClearClick(Sender: TObject);
begin
  h323.ClearCall(fToken, cerEndedByLocalUser);
end;

procedure TfrmDemo.btnDialClick(Sender: TObject);
begin
  h323.MakeCall(eDialNumber.Text);
end;

procedure TfrmDemo.btnAudioUpdateClick(Sender: TObject);
begin
  if h323.Active then
  begin
    h323.SoundChannelPlayDevice := ePlay.Text;  
    h323.SoundChannelRecordDevice := eRecord.Text;
    h323.AudioJitterDelayMin := StrToInt( eJitMin.Text );  
    h323.AudioJitterDelayMax := StrToInt( eJitMax.Text );
    h323.SoundChannelBufferDepth := StrToInt( eBufferDepth.Text );
  end;
end;

end.
